/**
 * RapidMiner GmbH
 *
 * Copyright (C) 2020 by RapidMiner GmbH and the contributors
 *
 * Complete list of developers available at our web site:
 *
 *      www.rapidminer.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package com.rapidminer.extension.shared_data.operator;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import com.rapidminer.connection.ConnectionInformation;
import com.rapidminer.connection.util.ConnectionInformationSelector;
import com.rapidminer.connection.util.ConnectionSelectionProvider;
import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DataRow;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.utils.ExampleSets;
import com.rapidminer.extension.shared_data.connection.SharedDataHandler;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.GenerateNewExampleSetMDRule;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.tools.Ontology;


/**
 * Simple operator that takes a {@link ConnectionInformation} as a parameter or input and outputs an {@link ExampleSet}
 * created from {@link SharedDataHandler#getConfiguration(ConnectionInformation, Operator)}  the configuration}.
 *
 * @since 1.0.0
 */
public class ReadSharedDataOperator extends Operator implements ConnectionSelectionProvider {

	private final OutputPort outputPort = getOutputPorts().createPort("example set");
	private ConnectionInformationSelector connectionSelector = new ConnectionInformationSelector(this, SharedDataHandler.SHARED_DATA_FULL_TYPE);

	public ReadSharedDataOperator(OperatorDescription description) {
		super(description);
		getTransformer().addRule(new GenerateNewExampleSetMDRule(outputPort));
		connectionSelector.makeDefaultPortTransformation();
		getTransformer().addRule(ConnectionInformationSelector.makeConnectionCheckTransformation(this));
	}

	@Override
	public void doWork() throws OperatorException {
		ConnectionInformation connection = connectionSelector.getConnection();
		Map<String, String> configuration = SharedDataHandler.getConfiguration(connection, this);

		List<Attribute> attributes = new ArrayList<>();
		configuration.forEach((k, v) -> attributes.add(AttributeFactory.createAttribute(k, Ontology.NOMINAL)));
		double[] data = new double[attributes.size()];
		for (int i = 0; i < attributes.size(); i++) {
			Attribute attribute = attributes.get(i);
			data[i] = attribute.getMapping().mapString(configuration.get(attribute.getName()));
		}
		DataRow dataRow = new DoubleArrayDataRow(data);
		ExampleSet output = ExampleSets.from(attributes).addDataRow(dataRow).build();
		connectionSelector.passDataThrough();
		outputPort.deliver(output);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		return new ArrayList<>(ConnectionInformationSelector.createParameterTypes(connectionSelector));
	}

	@Override
	public ConnectionInformationSelector getConnectionSelector() {
		return connectionSelector;
	}

	@Override
	public void setConnectionSelector(ConnectionInformationSelector selector) {
		// noop
	}

}
